#include "pch.h"
#include "VectorMath.h"
#include <vector>
#include <iostream>
#include <iomanip>
#include <sstream>

constexpr double pi = 3.14159265358979324;
constexpr double drag_c = 0.47;
constexpr double air_dense = 1.225;//kg/m^3
constexpr double x_area = 0.25;//m^2
constexpr double mass = 5.0;//kg

constexpr double launch_speed = 20.0;//m/s
constexpr double launch_angle = 40.0;//degrees
constexpr double launch_direction = 0.0;//degrees
constexpr double wind_speed = 10.0;//m/s
constexpr double wind_direction = 180.0;//degrees

using namespace std;

//angle unit conversion functions
double deg_to_rad(double degrees)
{
	double radians = pi * degrees / 180.0;
	while (radians >= 2 * pi) radians -= 2 * pi;
	while (radians < 0) radians += 2 * pi;
	return radians;
}
double rad_to_deg(double radians)
{
	double degrees = 180.0 * radians / pi;
	while (degrees >= 360.0) degrees -= 360.0;
	while (degrees < 0.0) degrees += 360.0;
	return degrees;
}

int main()
{
	//x-component is horizontal velocity along the original launch path
	//y-component is horizontal velocity perpendicular to original launch path
	//z-component is vertical velocity
	vector<double> velocity{ 0, 0, 0 };
	vector<double> position{ 0, 0, 0 };
	vector<double> wind{ 0, 0, 0 };
	vector<double> acceleration{ 0, 0, -9.8 };

	velocity[0] = launch_speed * cos(deg_to_rad(launch_angle)) * cos(deg_to_rad(launch_direction));//sets x-velocity m/s
	velocity[1] = launch_speed * cos(deg_to_rad(launch_angle)) * sin(deg_to_rad(launch_direction));//sets y-velocity m/s
	velocity[2] = launch_speed * sin(deg_to_rad(launch_angle));//sets z-velocity m/s
	double speed = launch_speed;//sets velocity

	wind[0] = wind_speed * cos(deg_to_rad(wind_direction));//sets x-velocity m/s
	wind[1] = wind_speed * sin(deg_to_rad(wind_direction));//sets y-velocity m/s

	double accel_drag; //initialization: how much speed lost due to drag

	double dt = 0.001;
	double air_time = 0;

	//loop simulates projectile motion in air
	do
	{
		//takes care of position change using velocity
		position = vector_add(position, scalar_multiplication(dt, velocity));
		air_time += dt;

		//defines acceleration due to drag and gravity
		vector<double> wind_relative_velocity = vector_subtract(wind, velocity);
		accel_drag = 0.5 * drag_c * air_dense * x_area * pow(magnitude(wind_relative_velocity), 2) / mass;
		acceleration = scalar_multiplication(accel_drag, normalize(wind_relative_velocity));
		acceleration[2] -= 9.8;

		//changes velocity due to acceleration
		velocity = vector_add(velocity, scalar_multiplication(dt, acceleration));
	} while (position[2] > 0);//repeat while hasn't hit ground yet

	//calculates landing position
	position.pop_back();
	double range = magnitude(position);
	double landing_direction = rad_to_deg(atan2(position[1], position[0]));

	//prepare output data
	stringstream ss;
	ss << setw(22) << "Mass: " << setw(10) << mass << " kilograms" << endl
		<< setw(22) << "Drag Coefficient: " << setw(10) << drag_c << endl
		<< setw(22) << "Air Density: " << setw(10) << air_dense << " kg/m^3" << endl
		<< setw(22) << "Cross-sectional Area: " << setw(10) << x_area << " m^2" << endl
		<< endl
		<< setw(22) << "Launch Speed: " << setw(10) << launch_speed << " m/s" << endl
		<< setw(22) << "Launch Angle: " << setw(10) << launch_angle << " degrees above horizontal" << endl
		<< setw(22) << "Launch Direction: " << setw(10) << launch_direction << " degrees" << endl
		<< setw(22) << "Wind Speed: " << setw(10) << wind_speed << " m/s" << endl
		<< setw(22) << "Wind Direction: " << setw(10) << wind_direction << " degrees" << endl
		<< endl
		<< setw(22) << "Range: " << setw(10) << range << " meters" << endl
		<< setw(22) << "Landing Direction: " << setw(10) << landing_direction << " degrees" << endl
		<< setw(22) << "Air Time: " << setw(10) << air_time << " seconds" << endl;
	cout << ss.str();
	return 0;
}